import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .datasources_and_workbooks_command import DatasourcesAndWorkbooks


class GetUrl(DatasourcesAndWorkbooks):
    """
    This command gets the resource from Tableau Server that's represented
    by the specified (partial) URL. The result is returned as a file.
    """

    name: str = "get"
    description: str = _("get.short_description")

    @staticmethod
    def define_args(get_url_parser):
        get_url_parser.add_argument("url", help=_("refreshextracts.options.url"))
        set_filename_arg(get_url_parser)
        # these don't need arguments, although that would be a good future addition
        # tabcmd get "/views/Finance/InvestmentGrowth.png?:size=640,480" -f growth.png
        # tabcmd get "/views/Finance/InvestmentGrowth.png?:refresh=yes" -f growth.png

    @staticmethod
    def run_command(args):
        # A view can be returned in PDF, PNG, or CSV (summary data only) format.
        # A Tableau workbook is returned as a TWB if it connects to a datasource/live connection,
        # or a TWBX if it uses an extract.
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        if " " in args.url:
            Errors.exit_with_error(logger, _("export.errors.white_space_workbook_view"))

        file_type = GetUrl.get_file_type_from_filename(logger, args.filename, args.url)
        content_type = GetUrl.evaluate_content_type(logger, args.url)
        if content_type == "workbook":
            if file_type == "twbx" or file_type == "twb":
                GetUrl.generate_twb(logger, server, args, file_type)
            else:
                Errors.exit_with_error(
                    logger, message=_("publish.errors.mutually_exclusive_option").format("twb", "twbx")
                )
        else:  # content type = view
            if file_type == "pdf":
                GetUrl.generate_pdf(logger, server, args)
            elif file_type == "png":
                GetUrl.generate_png(logger, server, args)
            elif file_type == "csv":
                GetUrl.generate_csv(logger, server, args)
            else:
                Errors.exit_with_error(logger, message=_("tabcmd.get.extension.not_found"))

    @staticmethod
    def evaluate_content_type(logger, url):
        # specify a view to get using "/views/<workbookname>/<viewname>.<extension>"
        # specify a workbook to get using "/workbooks/<workbookname>.<extension>".
        if url.find("/views/") == 0:
            return "view"
        elif url.find("/workbooks/") == 0:
            return "workbook"
        else:
            Errors.exit_with_error(logger, message=_("export.errors.requires_workbook_view_param").format(GetUrl.name))

    @staticmethod
    def get_file_type_from_filename(logger, file_name, url):
        type_of_file = None
        file_name = file_name or url
        logger.debug(_("get.options.file") + ": {}".format(file_name))
        type_of_file = GetUrl.get_file_extension(file_name)

        if not type_of_file:
            Errors.exit_with_error(logger, _("tabcmd.get.extension.not_found").format(file_name))
        else:
            logger.debug(_("get.options.file") + ": {}".format(type_of_file))
            if type_of_file in ["pdf", "csv", "png", "twb", "twbx"]:
                return type_of_file

        Errors.exit_with_error(logger, _("tabcmd.get.extension.not_found").format(file_name))

    @staticmethod
    def get_file_extension(filename):
        parts = filename.split(".")
        if len(parts) < 2:
            return None
        extension = parts[1]
        extension = GetUrl.strip_query_params(extension)
        return extension

    @staticmethod
    def strip_query_params(filename):
        if filename.find("?") > 0:
            filename = filename.split("?")[0]
        return filename

    @staticmethod
    def get_name_without_possible_extension(filename):
        if filename.find(".") > 0:
            filename = filename.split(".")[0]
        return filename

    @staticmethod
    def get_workbook_name(logger, url):  # /workbooks/wb-name" -> "wb-name"
        name_parts = url.split("/")
        if len(name_parts) != 3:
            raise ValueError(_("export.errors.requires_workbook_view_param").format(GetUrl.name))
        workbook_name = name_parts[::-1][0]  # last part
        workbook_name = GetUrl.strip_query_params(workbook_name)
        workbook_name = GetUrl.get_name_without_possible_extension(workbook_name)
        return workbook_name

    @staticmethod
    def get_view_url(url):  # "/views/wb-name/view-name" -> wb-name/sheets/view-name
        name_parts = url.split("/")  # ['', 'views', 'wb-name', 'view-name']
        if len(name_parts) != 4:
            raise ValueError(_("export.errors.requires_workbook_view_param").format(GetUrl.name))
        view_name = name_parts[::-1][0]
        view_name = GetUrl.strip_query_params(view_name)
        view_name = GetUrl.get_name_without_possible_extension(view_name)
        workbook_name = name_parts[2]
        return DatasourcesAndWorkbooks.get_view_url_from_names(workbook_name, view_name)

    @staticmethod
    def filename_from_args(file_argument, item_name, filetype):
        if file_argument is None:
            file_argument = "{}.{}".format(item_name, filetype)
        return file_argument

    @staticmethod
    def generate_pdf(logger, server, args):
        view_url = GetUrl.get_view_url(args.url)
        try:
            view_item: TSC.ViewItem = GetUrl.get_view_by_content_url(logger, server, view_url)
            logger.debug(_("content_type.view") + ": {}".format(view_item.name))
            req_option_pdf = TSC.PDFRequestOptions(maxage=1)
            server.views.populate_pdf(view_item, req_option_pdf)
            filename = GetUrl.filename_from_args(args.filename, view_item.name, "pdf")
            with open(filename, "wb") as f:
                f.write(view_item.pdf)
            logger.info(_("export.success").format(view_item.name, filename))
        except TSC.ServerResponseError as e:
            GetUrl.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)

    @staticmethod
    def generate_png(logger, server, args):
        view = GetUrl.get_view_url(args.url)
        try:
            view_item: TSC.ViewItem = GetUrl.get_view_by_content_url(logger, server, view)
            logger.debug(_("content_type.view") + ": {}".format(view_item.name))
            req_option_csv = TSC.CSVRequestOptions(maxage=1)
            server.views.populate_csv(view_item, req_option_csv)
            filename = GetUrl.filename_from_args(args.filename, view_item.name, "png")
            with open(filename, "wb") as f:
                f.write(view_item.png)
            logger.info(_("export.success").format(view_item.name, filename))
        except TSC.ServerResponseError as e:
            GetUrl.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)

    @staticmethod
    def generate_csv(logger, server, args):
        view_url = GetUrl.get_view_url(args.url)
        try:
            view_item: TSC.ViewItem = GetUrl.get_view_by_content_url(logger, server, view_url)
            logger.debug(_("content_type.view") + ": {}".format(view_item.name))
            req_option_csv = TSC.CSVRequestOptions(maxage=1)
            server.views.populate_csv(view_item, req_option_csv)
            file_name_with_path = GetUrl.filename_from_args(args.filename, view_item.name, "csv")
            with open(file_name_with_path, "wb") as f:
                f.write(view_item.csv)
            logger.info(_("export.success"), views_from_list.name, formatted_file_name)
        except TSC.ServerResponseError as e:
            GetUrl.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)
        except Exception as e:
            Errors.exit_with_error(logger, exception=e)

    @staticmethod
    def generate_twb(logger, server, args, file_extension):
        workbook_name = GetUrl.get_workbook_name(logger, args.url)
        try:
            target_workbook = GetUrl.get_wb_by_content_url(logger, server, workbook_name)
            logger.debug(_("content_type.workbook") + ": {}".format(workbook_name))
            file_name_with_path = GetUrl.filename_from_args(args.filename, workbook_name, file_extension)
            server.workbooks.download(target_workbook.id, filepath=file_name_with_path, no_extract=False)
            logger.info(_("export.success").format(target_workbook.name, file_name_with_path))
        except TSC.ServerResponseError as e:
            GetUrl.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)



from src.commands.auth.session import Session
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class LogoutCommand(Server):
    """
    Command to Log user out of the server
    """

    name: str = "logout"
    description: str = _("logout.short_description")

    @staticmethod
    def define_args(parser):
        # has no options
        pass

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        session.end_session_and_clear_data()



import tableauserverclient as TSC

from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _


class DatasourcesAndWorkbooks(Server):
    """
    Base Class for Operations related to Datasources and Workbooks
    """

    def __init__(self, args):
        super().__init__(args)

    @staticmethod
    def get_view_url_from_names(wb_name, view_name):
        return "{}/sheets/{}".format(wb_name, view_name)

    @staticmethod
    def get_view_by_content_url(logger, server, view_content_url) -> TSC.ViewItem:
        logger.debug(_("export.status").format(view_content_url))
        try:
            req_option = TSC.RequestOptions()
            req_option.filter.add(TSC.Filter("contentUrl", TSC.RequestOptions.Operator.Equals, view_content_url))
            matching_views, paging = server.views.get(req_option)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response").format(""))
        if len(matching_views) < 1:
            Errors.exit_with_error(logger, message=_("errors.xmlapi.not_found"))
        return matching_views[0]

    @staticmethod
    def get_wb_by_content_url(logger, server, workbook_content_url) -> TSC.WorkbookItem:
        logger.debug(_("export.status").format(workbook_content_url))
        try:
            req_option = TSC.RequestOptions()
            req_option.filter.add(TSC.Filter("contentUrl", TSC.RequestOptions.Operator.Equals, workbook_content_url))
            matching_workbooks, paging = server.workbooks.get(req_option)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response").format(""))
        if len(matching_workbooks) < 1:
            Errors.exit_with_error(logger, message=_("dataalerts.failure.error.workbookNotFound"))
        return matching_workbooks[0]



import pytest
import time
import subprocess
import unittest

from . import setup_e2e

debug_log = "--logging-level=DEBUG"
no_ssl = "--no-certcheck"
indexing_sleep_time = 1  # wait 1 second to confirm server has indexed updates

project_name = "not-default-name"
group_name = "test-ing-group"
workbook_name = "namebasic"

executable = ["python", "-m", "tabcmd"]
# if we want to test the pyinstaller exe
# executable = setup_e2e.Setup.get_executable()


@pytest.fixture(scope="module", autouse=True)
def log_in():
    setup_e2e.login()


# This is where you put tests that require server admin roles/on-prem server
def _test_command(test_args: list[str]):
    # this will raise an exception if it gets a non-zero return code
    # that should bubble up and fail the test?
    calling_args = executable + test_args + [debug_log] + [no_ssl]
    print(calling_args)
    return subprocess.check_call(calling_args)


class E2ETests(unittest.TestCase):
    def test_create_delete_group(self):
        command = "creategroup"
        arguments = [command, group_name]
        _test_command(arguments)

        time.sleep(1)

        command = "deletegroup"
        arguments = [command, group_name]
        _test_command(arguments)

    def test_create_delete_project(self):
        # project 1
        parent_path = project_name
        command = "createproject"
        arguments = [command, "--name", project_name]
        _test_command(arguments)

        time.sleep(indexing_sleep_time)

        # project 2
        parent_path = project_name
        command = "createproject"
        arguments = [command, "--name", project_name, "--parent-project-path", parent_path]
        _test_command(arguments)

        time.sleep(indexing_sleep_time)

        # project 3
        parent_path = "{0}/{1}".format(project_name, project_name)
        command = "createproject"
        arguments = [command, "--name", project_name, "--parent-project-path", parent_path]
        _test_command(arguments)

        time.sleep(indexing_sleep_time)

        # delete project 2 (containing 3)
        command = "deleteproject"
        arguments = [command, project_name, "--parent-project-path", project_name]
        _test_command(arguments)

        time.sleep(indexing_sleep_time)

        # delete project 1
        command = "deleteproject"
        arguments = [command, project_name]
        _test_command(arguments)

    def test_list_sites(self):
        command = "listsites"
        try:
            _test_command([command])
        except Exception as e:
            # often won't have permission for sites
            print(e)

    def test_create_extract(self):
        command = "createextracts"
        arguments = [command, "-d", "Regional", "--project", "Samples", "--encrypt"]
        _test_command(arguments)

    def test_publish_export(self):
        command = "publish"
        local_file = "tests/assets/SampleWB.twbx"
        arguments = [command, local_file]
        _test_command(arguments)
        # TODO: export and get the file we just published



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class EditSiteCommand(Server):
    """
    Command to change the name of a site or its web folder name. Users can also use this command to allow or deny
    site administrators the ability to add and remove users, or prevent users from running certain tasks manually.
    """

    name: str = "editsite"
    description: str = _("editsite.short_description")

    @staticmethod
    def define_args(edit_site_parser):
        edit_site_parser.add_argument("site_name", metavar="site-name", help="editsite.options.site-name")
        edit_site_parser.add_argument(
            "--site-name", default=None, dest="new_site_name", help=_("editsite.options.site-name")
        )

        set_common_site_args(edit_site_parser)
        set_site_status_arg(edit_site_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        site_item = Server.get_site_for_command_or_throw(logger, server, args)
        if args.url:
            site_item.content_url = args.url
        if args.user_quota:
            site_item.user_quota = args.user_quota
        if args.storage_quota:
            site_item.storage_quota = args.storage_quota
        if args.status:
            site_item.state = args.status
        try:
            logger.info(_("editsite.status").format(site_item.name))
            server.sites.update(site_item)
            logger.info(_("common.output.succeeded"))

        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)



import unittest

from src.commands.datasources_and_workbooks.delete_command import DeleteCommand
from .common_setup import *

commandname = "delete"


class DeleteParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteCommand)

    def test_delete_parser_no_object(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_delete_parser(self):
        mock_args = [commandname, "ds", "-r", "proj"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.name == "ds", args
        assert args.project_name == "proj", args

    def test_delete_parser_missing_args(self):
        mock_args = [commandname, "--datasource"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .user_data import UserCommand


class CreateSiteUsersCommand(UserCommand):
    """
    Command to add users to a site, based on information supplied in a comma-separated values (CSV) file.
    If the user is not already created on the server, the command creates the user before adding
    that user to the site
    """

    name: str = "createsiteusers"
    description: str = _("createsiteusers.short_description")

    @staticmethod
    def define_args(create_site_users_parser):
        set_role_arg(create_site_users_parser)
        set_users_file_positional(create_site_users_parser)
        set_completeness_options(create_site_users_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        number_of_users_listed = 0
        number_of_users_added = 0
        number_of_errors = 0

        creation_site = "current site"

        UserCommand.validate_file_for_import(args.filename, logger, detailed=True, strict=args.require_all_valid)

        logger.info(_("tabcmd.add.users.to_x").format(args.filename.name, creation_site))
        user_obj_list = UserCommand.get_users_from_file(args.filename, logger)
        logger.info(_("session.monitorjob.percent_complete").format(0))
        error_list = []
        for user_obj in user_obj_list:
            try:
                number_of_users_listed += 1
                result = server.users.add(user_obj)
                logger.info(_("tabcmd.result.success.create_user").format(user_obj.name))
                number_of_users_added += 1
            except TSC.ServerResponseError as e:
                number_of_errors += 1
                error_list.append(e)
                logger.debug(e)
        logger.info(_("session.monitorjob.percent_complete").format(100))
        logger.info(_("importcsvsummary.line.processed").format(number_of_users_listed))
        logger.info(_("importcsvsummary.line.skipped").format(number_of_errors))
        logger.info(_("importcsvsummary.users.added.count").format(number_of_users_added))
        if number_of_errors > 0:
            logger.info(_("importcsvsummary.error.details").format(error_list))



from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.commands.user.user_data import UserCommand
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class DeleteSiteUsersCommand(Server):
    """
    Command to Remove users from the site that user is logged in to.
    The users to be removed are specified in a file that contains
    a simple list of one user name per line.
    """

    name: str = "deletesiteusers"
    description: str = _("deletesiteusers.short_description")

    @staticmethod
    def define_args(delete_site_users_parser):
        set_users_file_positional(delete_site_users_parser)
        set_completeness_options(delete_site_users_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        logger.info(_("deleteusers.status").format(args.filename.name))

        UserCommand.validate_file_for_import(args.filename, logger, strict=args.require_all_valid)
        number_of_users_deleted = 0
        number_of_errors = 0
        user_obj_list = UserCommand.get_users_from_file(args.filename, logger)

        logger.debug("Users: {}".format(len(user_obj_list)))

        error_list = []
        for user_obj in user_obj_list:
            logger.info(_("importcsvsummary.line.processed").format(number_of_users_deleted))
            try:
                user_id = UserCommand.find_user_id(logger, server, user_obj.name)
                server.users.remove(user_id)
                logger.debug(_("tabcmd.result.success.delete_user").format(user_obj.name, user_id))
                number_of_users_deleted += 1
            except Exception as e:
                Errors.check_common_error_codes_and_explain(logger, e)
                number_of_errors += 1
                error_list.append(e)

        logger.info(_("importcsvsummary.line.processed").format(number_of_users_deleted))
        logger.info(_("importcsvsummary.errors.count").format(number_of_errors))
        if number_of_errors > 0:
            logger.info(_("importcsvsummary.error.details").format(error_list))



import logging

from .localize import set_client_locale
from .map_of_commands import *
from .parent_parser import ParentParser


class TabcmdController:
    @staticmethod
    def initialize():
        manager = ParentParser()
        parent = manager.get_root_parser()
        commands = CommandsMap.commands_hash_map
        for command in commands:
            manager.include(command)
        return parent

    # during normal execution, leaving input as none will default to sys.argv
    # for testing, we want to be able to pass in different updates
    @staticmethod
    def run(parser, user_input=None):
        if user_input is None and len(sys.argv) <= 1:  # no arguments given
            parser.print_help()
            sys.exit(0)
        user_input = user_input or sys.argv[1:]
        namespace = parser.parse_args(user_input)

        logger = log(__name__, namespace.logging_level or logging.INFO)
        logger.debug(namespace)
        if namespace.language:
            set_client_locale(namespace.language, logger)

        try:
            command_name = namespace.func
        except AttributeError as aer:
            # if no command was given, argparse will just not create the attribute
            parser.print_help()
            sys.exit(2)
        try:
            # if a subcommand was identified, call the function assigned to it
            # this is the functional equivalent of the call by reflection in the previous structure
            # https://stackoverflow.com/questions/49038616/argparse-subparsers-with-functions
            namespace.func.run_command(namespace)
        except Exception as e:
            logger.exception(e)

        return namespace



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class CreateExtracts(Server):
    """
    Command that creates extracts for a published workbook or data source.
    """

    name: str = "createextracts"
    description: str = _("createextracts.short_description")

    @staticmethod
    def define_args(create_extract_parser):
        set_ds_xor_wb_args(create_extract_parser)
        set_embedded_datasources_options(create_extract_parser)
        set_encryption_option(create_extract_parser)
        set_project_arg(create_extract_parser)
        set_parent_project_arg(create_extract_parser)
        set_site_url_arg(create_extract_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        creation_call = None
        try:
            logger.debug(
                "Extract params: encrypt={}, include_all={}, datasources={}".format(
                    args.encrypt, args.include_all, args.embedded_datasources
                )
            )

            if args.datasource:
                data_source_item = Server.get_data_source_item(logger, server, args.datasource)
                logger.info(_("createextracts.for.datasource").format(args.datasource))
                job = server.datasources.create_extract(data_source_item, encrypt=args.encrypt)

            elif args.workbook:
                workbook_item = Server.get_workbook_item(logger, server, args.workbook)
                logger.info(_("createextracts.for.workbook_name").format(args.workbook))
                job = server.workbooks.create_extract(
                    workbook_item,
                    encrypt=args.encrypt,
                    includeAll=args.include_all,
                    datasources=args.embedded_datasources,
                )
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, exception=e)

        logger.info(_("common.output.job_queued_success"))
        logger.debug("Extract creation queued with JobID: {}".format(job.id))



import unittest
from unittest import mock

from src.commands.server import Server
from src.execution.logger_config import log

fake_item = mock.MagicMock()
fake_item.name = "fake-name"
fake_item.id = "fake-id"
getter = mock.MagicMock("get", return_value=([fake_item], 1))


class ProjectsTest(unittest.TestCase):
    logger = log("Projects_Tests", "debug")

    @staticmethod
    def test_parent_path_to_list():
        assert Server._parse_project_path_to_list(None) == []
        assert Server._parse_project_path_to_list("") == [""]
        assert Server._parse_project_path_to_list("parent") == ["parent"]
        assert Server._parse_project_path_to_list("parent/child") == ["parent", "child"]

    @mock.patch("tableauserverclient.Server")
    def test_get_project(self, mock_server):
        mock_server.projects.get = getter
        Server.get_project_by_name_and_parent_path(mock.MagicMock(), mock_server, "random_name", "")
        getter.assert_called()



import unittest
import logging
from src.commands.constants import Errors


# TODO add checks that the logger was called?
class ServerTests(unittest.TestCase):
    def test_exit_with_logger_problem(self):
        try:
            Errors.exit_with_error(None, "a message appears")
        except SystemExit as um:
            assert um.code == 1

    def test_exit_with_message(self):
        try:
            log = logging.getLogger("exitwithmessage")
            Errors.exit_with_error(log, "a message appears")
        except SystemExit as um:
            assert um.code == 1

    def test_exit_with_exception(self):
        try:
            log = logging.getLogger("exitwithexception")
            Errors.exit_with_error(log, "a message appears")
        except SystemExit as um:
            assert um.code == 1



import unittest

from src.commands.extracts.decrypt_extracts_command import DecryptExtracts
from .common_setup import *

commandname = "decryptextracts"


class DecryptExtractsParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DecryptExtracts)

    def test_decrypt_extract_parser_required_name(self):
        mock_args = [commandname, "site-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "site-name", args

    """
    bug: the site name is supposed to be optional
    def test_decrypt_extract_parser_missing_site_name(self):
        mock_args = [commandname]
        args = self.parser_under_test.parse_args(mock_args)
    """



import unittest

from src.commands.extracts.delete_extracts_command import DeleteExtracts
from .common_setup import *

commandname = "deleteextracts"


class DeleteExtractsParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteExtracts)

    def test_delete_extract_parser_datasource(self):
        mock_args = [
            commandname,
            "-d",
            "ds-name",
            "--project",
            "prjt",
            "--parent-project-path",
            "ppp",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.datasource == "ds-name", args
        assert args.project_name == "prjt", args
        assert args.parent_project_path == "ppp"

    def test_delete_extract_parser_workbook(self):
        mock_args = [commandname, "-w", "wb-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.workbook == "wb-name", args
        assert args.project_name == "", args

    def test_delete_extract_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_delete_extract_parser_ds_and_wb(self):
        mock_args = [commandname, "-d", "ds", "-w", "wb"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import gettext
import locale
import logging
import os
import sys
from typing import Any
from typing import Callable

translate = None


# wrapper around the gettext _ function to load it on demand if required
def _(string_key: str) -> str:
    global translate
    if not translate:
        translate = set_client_locale(None, None)
    if not translate:
        translate = _identity_func
    return translate(string_key)


def _identity_func(x: Any) -> Any:
    return x


# The client should present text in the OS language, or english if not present.
def set_client_locale(lang: str = None, logger=None) -> Callable:
    if not logger:
        logger = logging.getLogger()

    global translate
    try:
        locale_options = [_validate_lang(lang), _get_default_locale(), "en"]
    except Exception as e:
        print(e)
        locale_options = ["en"]

    logger.debug("Language options: {}".format(locale_options))

    domain = "tabcmd"
    for lang in locale_options:
        try:
            if lang:
                translate = _load_language(lang, domain)
                break
        except Exception as e:
            print("Failed to load language '", lang, "':", e)

    return translate or _identity_func


# Handling file locations in unbundled (e.g dev) layout and when bundled by pyinstaller
# https://stackoverflow.com/questions/7674790/bundling-data-files-with-pyinstaller-onefile/13790741#13790741
def resource_path(relative_path):
    """Get absolute path to resource, works for dev and for PyInstaller"""
    base_path = getattr(sys, "_MEIPASS", os.path.dirname(os.path.abspath(__file__)))
    return os.path.join(base_path, relative_path)


def _load_language(current_locale, domain):
    locale_path = os.path.join("..", "locales")

    # fallback=True means if loading the translated files fails, strings will be returned
    # we use the identity function above instead
    locale_dir = resource_path(locale_path)
    language: gettext.NullTranslations = gettext.translation(
        domain, locale_dir, languages=[current_locale], fallback=False
    )
    language.install()  # I believe this is the expensive call
    _ = language.gettext
    return _


def _get_default_locale():
    current_locale, encoding = locale.getdefaultlocale()
    current_locale = _validate_lang(current_locale)
    return current_locale


def _validate_lang(requested_locale):
    if not requested_locale:
        return None
    if requested_locale in ["en_GB", "fr_CA", "zh_TW"]:
        return requested_locale
    else:
        abbreviated_locale = requested_locale.split("_")[0]
        if abbreviated_locale in ["de", "es", "fr", "ga", "it", "ja", "ko", "pt", "sv", "zh"]:
            return abbreviated_locale
    return None



import unittest
from unittest import mock
from src.commands.datasources_and_workbooks.get_url_command import *
from src.commands.datasources_and_workbooks.export_command import *
from src.commands.server import Server

mock_logger = mock.MagicMock()


class GeturlTests(unittest.TestCase):
    def test_evaluate_file_name_pdf(self):
        filename = "filename.pdf"
        url = None
        filetype = GetUrl.get_file_type_from_filename(mock_logger, filename, url)
        assert filetype == "pdf", filetype

    def test_evaluate_file_name_url(self):
        filename = "filename.twbx"
        filetype = Server.get_filename_extension_if_tableau_type(mock_logger, filename)
        assert filetype == "twbx", filetype

    def test_evaluate_file_path(self):
        filename = "../assets/filename.twb"
        filetype = Server.get_filename_extension_if_tableau_type(mock_logger, filename)
        assert filetype == "twb", filetype

    def test_evaluate_file_name_url_pdf_fails(self):
        filename = "filename.pdf"
        with self.assertRaises(ValueError):
            filetype = Server.get_filename_extension_if_tableau_type(mock_logger, filename)

    def test_evaluate_file_name_url_no_ext_fails(self):
        filename = "filename/filename/filename/file"
        with self.assertRaises(ValueError):
            filetype = Server.get_filename_extension_if_tableau_type(mock_logger, filename)

    def test_get_view_without_extension_that_does_have_one(self):
        filename = "viewname.pdf"
        assert GetUrl.get_name_without_possible_extension(filename) == "viewname"

    def test_get_view_without_extension_that_doesnt_have_one(self):
        filename = "viewname"
        assert GetUrl.get_name_without_possible_extension(filename) == filename

    def test_get_workbook_name(self):
        assert GetUrl.get_workbook_name(mock_logger, "/workbooks/wbname") == "wbname"

    def test_view_name(self):
        assert GetUrl.get_view_url("/views/wb-name/view-name") == "wb-name/sheets/view-name"

    """
    GetUrl.get_view_without_extension(view_name)
    GetUrl.get_view(url)
    GetUrl.get_workbook(url)
    GetUrl.generate_twb(logger, server, args)
    GetUrl.generate_pdf(logger, server, args)
    GetUrl.generate_png(logger, server, args)
    GetUrl.generate_csv(logger, server, args)
    """


class ExportTests(unittest.TestCase):
    def test_parse_export_url_to_workbook(self):
        wb_url = "wb-name/view-name"
        view, wb = ExportCommand.parse_export_url_to_workbook_and_view(mock_logger, wb_url)
        assert view == "wb-name/sheets/view-name"
        assert wb == "wb-name"



import unittest
from unittest import mock

from src.commands.user.add_users_command import AddUserCommand
from .common_setup import *

commandname = "addusers"


class AddUsersParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, AddUserCommand)

    def test_add_users_parser_role(self):
        cmd_line_input = [commandname, "group-name", "--users", "users.csv"]
        with mock.patch("builtins.open", mock.mock_open(read_data="users.csv")) as file:
            args_from_command = vars(self.parser_under_test.parse_args(cmd_line_input))
            print(args_from_command)
            assert args_from_command["users"] is not None, args_from_command
            assert args_from_command["func"] is not None, args_from_command  # has id'd a subcommand
            assert args_from_command["name"] == "group-name", args_from_command
            assert args_from_command["require_all_valid"] is True, args_from_command

    def test_add_users_parser_users_file(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")) as open_file:
            mock_args = [commandname, "group-name", "--users", "users.csv"]
            args = self.parser_under_test.parse_args(mock_args)
            self.assertEqual(args.name, "group-name"), args
            open_file.assert_called_with("users.csv", "r", -1, "UTF-8", None), args

    @mock.patch("builtins.open")
    def test_add_user_parser_missing_group_name(self, filereader):
        cmd_line_input = [commandname, "--users", "test_csv.csv"]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(cmd_line_input)

    def test_add_user_parser_missing_users(self):
        cmd_line_input = [commandname, "group-name"]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(cmd_line_input)

    @mock.patch("builtins.open")
    def test_add_user_parser_complete(self, filereader):
        cmd_line_input = [
            commandname,
            "group-name",
            "--users",
            "users.csv",
            "--complete",
        ]
        with mock.patch("builtins.open", mock.mock_open(read_data="users.csv")) as file:
            args_from_command = vars(self.parser_under_test.parse_args(cmd_line_input))
            assert args_from_command["require_all_valid"] is True, args_from_command

    @mock.patch("builtins.open")
    def test_add_user_parser_extra_args_present(self, filereader):
        cmd_line_input = [commandname, "group-name", "--users", "users.csv", "what"]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(cmd_line_input)



from setuptools import setup, find_packages


setup(
    name='tabcmd',
    author='Tableau',
    author_email='github@tableau.com',
    description='A command line client for working with Tableau Server.',
    long_description='A command line client for working with Tableau Server.',
    license='MIT',
    url='https://github.com/tableau/tabcmd',

    python_requires='>=3.7',
    packages=find_packages(),
    package_data={'tabcmd': ['src.locales/**/*.mo']},
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'tabcmd = src.tabcmd:main'
        ]
    },
    install_requires=[
        'polling2',
        'requests>=2.11,<3.0',
        'setuptools>=45',
        'tableauserverclient>=0.19',
        'urllib3>=1.24.3,<2.0',
    ],
    test_suite='tests',
    extras_require={
        'build': [
            'appdirs',
            'black',
            'mypy',
            'pyinstaller_versionfile',
            'setuptools_scm',
            'types-appdirs',
            'types-mock',
            'types-requests',
        ],
        'package': [
            'pyinstaller>=5.1',
            'pyinstaller-versionfile',
        ],
        'test': [
            'mock',
            'pytest', 'pytest-cov', 'pytest-order', 'pytest-runner',
            'requests-mock>=1.0,<2.0',
        ]
    },
    use_scm_version={
        "write_to": "src/execution/_version.py",
        "local_scheme": "no-local-version"  # require pypi supported versions always
    },
    zip_safe=False,
)



from src.commands.auth.login_command import *
from src.commands.auth.logout_command import *
from src.commands.datasources_and_workbooks.delete_command import *
from src.commands.datasources_and_workbooks.export_command import *
from src.commands.datasources_and_workbooks.get_url_command import *
from src.commands.datasources_and_workbooks.publish_command import *
from src.commands.extracts.create_extracts_command import *
from src.commands.extracts.decrypt_extracts_command import *
from src.commands.extracts.delete_extracts_command import *
from src.commands.extracts.encrypt_extracts_command import *
from src.commands.extracts.reencrypt_extracts_command import *
from src.commands.extracts.refresh_extracts_command import *
from src.commands.group.create_group_command import *
from src.commands.group.delete_group_command import *
from src.commands.help.help_command import *
from src.commands.project.create_project_command import *
from src.commands.project.delete_project_command import *
from src.commands.project.publish_samples_command import *
from src.commands.site.create_site_command import *
from src.commands.site.delete_site_command import *
from src.commands.site.edit_site_command import *
from src.commands.site.list_sites_command import *
from src.commands.user.add_users_command import *
from src.commands.user.create_site_users import *
from src.commands.user.delete_site_users_command import *

# from src.commands.user.create_users import *
from src.commands.user.remove_users_command import *


class CommandsMap:
    commands_hash_map = [
        # not yet implemented "createusers": ("createusers", CreateUserCommand, "Create users on the server"),
        # run schedule
        AddUserCommand,
        CreateExtracts,
        CreateGroupCommand,
        CreateProjectCommand,
        CreateSiteCommand,
        CreateSiteUsersCommand,
        DecryptExtracts,
        DeleteCommand,
        DeleteExtracts,
        DeleteGroupCommand,
        DeleteProjectCommand,
        DeleteSiteCommand,
        DeleteSiteUsersCommand,
        EditSiteCommand,
        EncryptExtracts,
        ExportCommand,
        GetUrl,
        HelpCommand,
        ListSiteCommand,
        LoginCommand,
        LogoutCommand,
        PublishCommand,
        PublishSamplesCommand,
        ReencryptExtracts,
        RefreshExtracts,
        RemoveUserCommand,
    ]



# generates a version data file that will be read by pyinstaller
import pyinstaller_versionfile
import sys
import os
import re

# TODO move this to the do-it scripts

# this file contains the actual version, generated by setuptools_scm
mymodule_dir = os.path.join(os.path.dirname( __file__ ), '..')
sys.path.append(mymodule_dir)
from src.execution._version import version

# setuptools produces a version like '0.1.dev185+345'
# pyinstaller requires the version be numeric only
print(version)
intermediate = (re.sub(r'[\+]', '.', version.lower())).__str__()
numeric_version = (re.sub(r'[a-z+]', '', intermediate.lower())).__str__()
print(numeric_version)

output_file = os.path.join(mymodule_dir, "versionfile.txt")
print(output_file)
input_file = os.path.join(mymodule_dir, 'res', "metadata.yml")
pyinstaller_versionfile.create_versionfile_from_input_file(
    output_file, input_file,
    # optional, can be set to overwrite version information (equivalent to --version when using the CLI)
    version=numeric_version
)



import unittest

from src.commands.group.delete_group_command import DeleteGroupCommand
from .common_setup import *

commandname = "deletegroup"


class DeleteGroupParserTestT(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteGroupCommand)

    def test_delete_group(self):
        mock_args = [commandname, "group-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.name == "group-name", args

    def test_delete_group_parser_required_name_missing(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import logging
import os

path = os.path.dirname(os.path.abspath(__file__))

FORMATS = {
    logging.ERROR: "ERROR: %(name)-10s: %(lineno)d: %(message)s",
    logging.WARN: "WARN:  %(message)s",
    logging.DEBUG: "DEBUG: %(name)-10s: %(lineno)d: %(message)-10s",
    logging.INFO: "INFO:  %(message)s",
    "TRACE": "TRACE: %(asctime)-12s %(name)-10s: %(lineno)d: %(message)-10s",
    "DEFAULT": "%(message)s",
}


def configure_log(name: str, logging_level_input: str):
    """function for logging statements to console and logfile"""
    logging_level = getattr(logging, logging_level_input.upper())
    log_format = FORMATS[logging_level]
    logging.basicConfig(
        level=logging_level, format=log_format, filename="tabcmd.log", filemode="a", datefmt="%Y-%m" "-%d " "%H:%M:%S"
    )
    console = logging.StreamHandler()
    console.setLevel(logging_level)
    console.setFormatter(logging.Formatter(log_format))
    logging.getLogger(name).addHandler(console)
    return logging.getLogger(name)


def log(file_name, logging_level):
    logger = configure_log(file_name, logging_level)
    return logger



import unittest

from src.commands.site.edit_site_command import EditSiteCommand
from .common_setup import *

commandname = "editsites"


class EditSiteParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, EditSiteCommand)

    def test_edit_site_parser_optional_args_present(self):
        mock_args = [
            commandname,
            "site-to-edit",
            "--site-name",
            "new-site-name",
            "--user-quota",
            "12",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "site-to-edit", args
        assert args.new_site_name == "new-site-name", args
        assert args.user_quota == 12, args

    def test_edit_site_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    """
    bug: this should probably exit unhappy?
    def test_edit_site_parser_missing_all_args(self):
        mock_args = [commandname, 'site-to-edit']
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)
    """

    def test_edit_site_parser_storage_quota_integer(self):
        mock_args = [commandname, "site-to-edit", "--storage-quota", "12"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "site-to-edit", args
        assert args.storage_quota == 12, args

    def test_edit_site_parser_optional_arguments_archive(self):
        mock_args = [
            commandname,
            "site-to-edit",
            "--status",
            "ACTIVE",
            "--site-id",
            "1234",
            "--run-now-enabled",
            "true",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_id == "1234", args
        assert args.status == "ACTIVE", args
        assert args.run_now_enabled == "true", args



from src.commands.auth.session import Session
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .user_data import UserCommand


class AddUserCommand(UserCommand):
    """
    Command to Adds users to a specified group
    """

    name: str = "addusers"
    description: str = _("addusers.short_description")

    @staticmethod
    def define_args(add_user_parser):
        add_user_parser.add_argument("name", help="name of group to add users to")
        set_users_file_arg(add_user_parser)
        set_completeness_options(add_user_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        logger.info(_("tabcmd.add.users.to_site").format(args.users.name, args.name))

        UserCommand.act_on_users(logger, server, "added", server.groups.add_user, args)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class ReencryptExtracts(Server):
    """
    Command to Reencrypt all extracts on a site with new encryption keys.
    This command will regenerate the key encryption key and data encryption key. You must specify a site.
    """

    name: str = "reencryptextracts"
    description: str = _("reencryptextracts.short_description=")

    @staticmethod
    def define_args(reencrypt_extract_parser):
        reencrypt_extract_parser.add_argument("site_name", metavar="site-name", help=_("editsite.options.site-name"))

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        site_item = Server.get_site_for_command_or_throw(logger, server, args)
        try:
            logger.info(_("reencryptextracts.status").format(site_item.name))
            job = server.sites.encrypt_extracts(site_item.id)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, e)

        logger.info(_("common.output.job_queued_success"))
        logger.debug("Extract re-encryption queued with JobID: {}".format(job.id))



import unittest

from src.commands.extracts.refresh_extracts_command import RefreshExtracts
from .common_setup import *

commandname = "refreshextracts"


class RefreshExtractsParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, RefreshExtracts)

    def test_refresh_extract_parser_conflicting_arguments(self):
        mock_args = [commandname, "--datasource", "hello", "--workbook", "testworkbook"]  # --url
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_refresh_extract_parser_optional_arguments(self):
        mock_args = [
            commandname,
            "--datasource",
            "hello",
            "--incremental",
            "True",
            "--removecalculations",
            "--project",
            "testproject",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.incremental == "True", args

    def test_refresh_extract_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import unittest
from unittest import mock

from src.commands.auth.login_command import LoginCommand
from .common_setup import *

commandname = "login"


@mock.patch("sys.argv", None)
class LoginParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, LoginCommand)

    def test_Login_username_password(self):
        mock_args = [
            commandname,
            "--server",
            "the-server",
            "--site",
            "the-site",
            "--username",
            "me",
            "--password",
            "the-password",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.username == "me", args
        assert args.password == "the-password", args
        assert args.server == "the-server", args
        assert args.site_name == "the-site"

    def test_parsing_connection_settings(self):
        mock_args = [commandname, "--no-certcheck", "--proxy", "the-proxy", "--timeout", "10"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.no_certcheck is True, args
        assert args.proxy == "the-proxy", args
        assert args.timeout == "10", args

    def test_parsing_no_proxy_no_cert(self):
        mock_args = [commandname, "--no-certcheck", "--no-proxy"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.no_certcheck, args
        assert not args.proxy, args

    def test_parsing_cert(self):
        mock_args = [commandname, "--use-certificate", "certificate-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert not args.no_certcheck, args
        assert args.certificate == "certificate-name"

    def test_parsing_conflicting_cert_args(self):
        mock_args = [commandname, "--no-certcheck", "-c", "cert-name"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_parsing_conflicting_proxy_args(self):
        mock_args = [commandname, "--no-proxy", "-x", "proxy-name"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_server_not_required(self):
        mock_args = [commandname, "--logging-level", "DEBUG", "--username", "me"]
        args = self.parser_under_test.parse_args(mock_args)
        assert not args.server, args
        assert args.logging_level == "DEBUG", args

    def test_username_not_required(self):
        mock_args = [commandname, "--server", "the-server"]
        args = self.parser_under_test.parse_args(mock_args)

    def test_token_username_conflict(self):
        mock_args = [commandname, "--username", "the-user", "--token-name", "a-token"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_password_password_file_conflict(self):
        mock_args = [commandname, "--password", "the-pwd", "--password_file", "a-file-name"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_version(self):
        mock_args = [commandname, "--version", "the-pwd", "--password_file", "a-file-name"]
        with self.assertRaises(SystemExit):
            # this runs an action and exits
            args = self.parser_under_test.parse_args(mock_args)

    def test_login_parser_test_username_password(self):
        mock_args = [commandname, "--username", "helloworld", "--password", "pw"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.username == "helloworld", args
        assert args.password == "pw", args

    def test_login_parser_test_token(self):
        mock_args = [commandname, "--token", "token", "--token-name", "tn"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.token_name == "tn", args
        assert args.token == "token", args

    def test_login_token_and_username(self):
        mock_args = [commandname, "--token-name", "to", "--username", "u"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)
